/*
portable.h
Copyright (C)2004 Al Kingsley

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Don't write to me at: kingsley@bananahead.com 

	Part of:
	BhPos, Copyright (C) 2001-2005 Al Kingsley
	BhPos comes with ABSOLUTELY NO WARRANTY.
*/

#ifndef _PORTABLE_H_
#define _PORTABLE_H_

#include <list>
#include <bhdev.h>

namespace bhpos{ 

/**
 * @defgroup bhportable Portable Barcode Scanner
 * @{
 *
 * \ingroup hwlib
 * @{	
 */
   
enum PortableDeviceType{
   PORTDEV_TYPE_NONE,
   PORTDEV_TYPE_KEYBOARD,
   PORTDEV_TYPE_SERIAL,
   PORTDEV_TYPE_USB
};
   
#define BHPORTABLE_TYPE_NONE   0
#define BHPORTABLE_TYPE_SYMBOL 1

enum BHPT_SYM_OPCODE {
   AIM_ON, 
   AIM_OFF, 
   BATCH_REQ,
   BATCH_DATA,
   BEEP,
   CMD_ACK, 
   CMD_NAK, 
   DECODE_DATA, 
   EVENT, 
   LED_OFF, 
   LED_ON, 
   PARAM_DEFAULTS, 
   PARAM_REQUEST, 
   PARAM_SEND,
   REPLY_REVISION, 
   REQUEST_REVISION, 
   SCAN_DISABLE, 
   SCAN_ENABLE, 
   SLEEP,
   START_DECODE, STOP_DECODE,
   WAKEUP
};

#define BHPORTABLE_SYM_OPCODE_FLUSHMACRO  0x10
#define BHPORTABLE_SYM_OPCODE_ABORTMACRO  0x11
#define BHPORTABLE_SYM_OPCODE_BATCHREQ    0xD5
#define BHPORTABLE_SYM_OPCODE_BATCHDATA   0xD6
#define BHPORTABLE_SYM_OPCODE_CMDACK      0xD0
#define BHPORTABLE_SYM_OPCODE_CMDNAK      0xD1
   
#define BHPORTABLE_SYM_MSGSRC_DECODER     0x00
#define BHPORTABLE_SYM_MSGSRC_HOST        0x04

#define BHPORTABLE_SYM_STATUS_LASTPACKET  0x01
#define BHPORTABLE_SYM_STATUS_CONTPACKET  0x02

/*! \brief Symbol packet. */
typedef struct tagsymbol_packet{
   unsigned char  length;
   unsigned char  opcode;
   unsigned char  msg_src;
   unsigned char  status;
   char  msg[251];
   unsigned char  chksum_hi;
   unsigned char  chksum_lo;
}symbol_packet;

/*! \brief Symbol barcode. */
typedef struct tagsymbol_barcode{
   unsigned char size;
   unsigned char type;
   char data[16];
}symbol_barcode;

typedef struct portable_barcode{
   union {
      char           raw[18];
      symbol_barcode symbol;
   } data;
}portable_barcode;

/*! \brief A class to portable barcode scanners.
 */

class CPortable : public CBhDevice
{
private:
   int             dev;
   int             j;
   symbol_packet   pkt;
   symbol_barcode  bc;
   std::list<symbol_barcode>  bc_list;
	
protected:
   unsigned char FetchChar();
   int  BatchReq();
   int  GetPacket();
   bool CheckSumOk(unsigned short);

public:
	CPortable(BhDevType);
	~CPortable();
	int  OpenScanner(int, BhDevNotifyType);
   bool InCradle();
   int  LoadBarcodes();
	int  GetNumBarcodes();
   int  PopBarcode(symbol_barcode *sbc);
   int  PushBarcode(symbol_barcode &sbc);
};
/* }@ 
	}@ */
}

#endif //_PORTABLE_H_
