/*
cprinter.h
Copyright (C)2004 Al Kingsley

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Don't write to me at: kingsley@bananahead.com 

	Part of:
	BhPos, Copyright (C) 2001-2005 Al Kingsley
	BhPos comes with ABSOLUTELY NO WARRANTY.
*/

#ifndef _CPRINTER_H_
#define _CPRINTER_H_
#include <libxml++/libxml++.h>
#include <ctransitems.h>
#include <bhdev.h>

namespace bhpos{ 

enum PrinterDeviceType{
   PRNDEV_TYPE_NONE,
   PRNDEV_TYPE_PARALLEL,
   PRNDEV_TYPE_SERIAL,
   PRNDEV_TYPE_USB,
   PRNDEV_TYPE_NETWORK
};

//#include "postdev.h"
//#include "pos.h"

/**
 * @defgroup printer Printer
 * @{
 *
 * @ingroup hwlib
 * @{	
 */

#define PRINTER_PARAM_HDRMSG		1
#define PRINTER_PARAM_FTRMSG		2
#define PRINTER_PARAM_SLIPAREA	3

#define PRINTER_FEATURE_BASE				0x00000001
#define PRINTER_FEATURE_MICR			  (PRINTER_FEATURE_BASE)
#define PRINTER_FEATURE_MICRE13B	     (PRINTER_FEATURE_BASE << 1)
#define PRINTER_FEATURE_MICRCMC7	     (PRINTER_FEATURE_BASE << 2)
#define PRINTER_FEATURE_CUSTDISP	 	  (PRINTER_FEATURE_BASE << 3)
#define PRINTER_FEATURE_CASHDRAWER	  (PRINTER_FEATURE_BASE << 4)

/*
#define PRINTER_DEVSTATUS_CLOSED		0
#define PRINTER_DEVSTATUS_OPEN		1
#define PRINTER_DEVSTATUS_READERR	2
#define PRINTER_DEVSTATUS_WRITEERR 3
*/

/** 
  \brief
 */
enum BhPrinterStatusMethod{
	PRN_STATUSMETHOD_NORMAL,
	PRN_STATUSMETHOD_REALTIME,
	PRN_STATUSMETHOD_MICRREALTIME,
	PRN_STATUSMETHOD_ASB
};

/** 
  \brief
 */
enum BhPrinterStatusType{
   PRN_STATUSTYPE_NONE,
	PRN_STATUSTYPE_SLIPSELECT,
	PRN_STATUSTYPE_SLIPLOAD,
   PRN_STATUSTYPE_SLIPALL,
   PRN_STATUSTYPE_SLIPREMOVE,
   PRN_STATUSTYPE_CHEQUESELECT,
	PRN_STATUSTYPE_CHEQUELOAD,
   PRN_STATUSTYPE_CHEQUEALL,
   PRN_STATUSTYPE_CHEQUEREMOVE,
	PRN_STATUSTYPE_MICRSELECT,
	PRN_STATUSTYPE_MICRLOAD,
   PRN_STATUSTYPE_MICRALL,
   PRN_STATUSTYPE_MICRREMOVE,
   PRN_STATUSTYPE_CASHDRAWERCLOSED,
   PRN_STATUSTYPE_CASHDRAWEROPENCLOSED
};

/** 
  \brief
 */
enum BhPrinterStatusEvent{
	PRN_STATUSEVENT_SLIPSELECTED,
	PRN_STATUSEVENT_SLIPLOADED,
   PRN_STATUSEVENT_SLIPCANCELLED,
   PRN_STATUSEVENT_SLIPREMOVED,
   PRN_STATUSEVENT_CHEQUESELECTED,
	PRN_STATUSEVENT_CHEQUELOADED,
   PRN_STATUSEVENT_CHEQUECANCELLED,
   PRN_STATUSEVENT_CHEQUEREMOVED,
	PRN_STATUSEVENT_MICRSELECTED,
	PRN_STATUSEVENT_MICRLOADED,
   PRN_STATUSEVENT_MICRREAD,
   PRN_STATUSEVENT_MICRCANCELLED,
   PRN_STATUSEVENT_MICRREMOVED,
   PRN_STATUSEVENT_CASHDRAWEROPEN,
   PRN_STATUSEVENT_CASHDRAWERCLOSED
};

/** 
  \brief
 */
enum BhPrinterErrorEvent{
	PRN_ERROREVENT_SLIPSELECTED,
	PRN_ERROREVENT_SLIPLOADED,
   PRN_ERROREVENT_SLIPCANCELLED,
   PRN_ERROREVENT_SLIPREMOVED,
   PRN_ERROREVENT_CHEQUESELECTED,
	PRN_ERROREVENT_CHEQUELOADED,
   PRN_ERROREVENT_CHEQUECANCELLED,
   PRN_ERROREVENT_CHEQUEREMOVED,
	PRN_ERROREVENT_MICRSELECTED,
	PRN_ERROREVENT_MICRLOADED,
   PRN_ERROREVENT_MICRREAD,
   PRN_ERROREVENT_MICRCANCELLED,
   PRN_ERROREVENT_MICRREMOVED
};

#define PRINTER_STATUS_UNKNOWN         0x00000000
#define PRINTER_GETSTATUS_PRINTER      0x00000001
#define PRINTER_GETSTATUS_OFFLINE      (0x00000001 << 1)
#define PRINTER_GETSTATUS_ERROR        (0x00000001 << 2)
#define PRINTER_GETSTATUS_PAPERROLL    (0x00000001 << 3)
#define PRINTER_GETSTATUS_SLIP         (0x00000001 << 4)
#define PRINTER_GETSTATUS_VALIDATION   (0x00000001 << 5)
#define PRINTER_GETSTATUS_MICR         (0x00000001 << 6)

// from status n=1
#define PRINTER_STATUS_KICK		 	    0x00000001
#define PRINTER_STATUS_ONLINE				(0x00000001 << 1)
#define PRINTER_STATUS_WOLRECOVERY     (0x00000001 << 3)
#define PRINTER_STATUS_PAPERFEED	      (0x00000001 << 3)
// from status n=2
#define PRINTER_STATUS_COVER   		   (0x00000001 << 4)
#define PRINTER_STATUS_OLPAPERFEED     (0x00000001 << 5)
#define PRINTER_STATUS_PAPEREND	      (0x00000001 << 6)
#define PRINTER_STATUS_ERROR	 	      (0x00000001 << 7)
// from status n=3
#define PRINTER_STATUS_MECHERROR  	   (0x00000001 << 8)
#define PRINTER_STATUS_ACUTTERERROR    (0x00000001 << 9)
#define PRINTER_STATUS_UNRECERROR	   (0x00000001 << 10)
#define PRINTER_STATUS_AUNRECERROR	   (0x00000001 << 11)
// from status n=4
#define PRINTER_STATUS_NEAREND    	   (0x00000001 << 12)
#define PRINTER_STATUS_PAPERPRESENT    (0x00000001 << 13)
// from status n=5
#define PRINTER_STATUS_SLIPSELECT	   (0x00000001 << 14)
#define PRINTER_STATUS_SLIPWAIT   	   (0x00000001 << 15)
#define PRINTER_STATUS_SLIPTOF    	   (0x00000001 << 16)
#define PRINTER_STATUS_SLIPBOF    	   (0x00000001 << 17)
// from status n=6
#define PRINTER_STATUS_VALSELECT	      (0x00000001 << 18)
#define PRINTER_STATUS_VALWAIT    	   (0x00000001 << 19)
#define PRINTER_STATUS_VALTOF     	   (0x00000001 << 20)
#define PRINTER_STATUS_VALBOF     	   (0x00000001 << 21)
// from status mirc
#define PRINTER_STATUS_MICRSELECT	   (0x00000001 << 22)
#define PRINTER_STATUS_MICRWAIT    	   (0x00000001 << 23)
#define PRINTER_STATUS_MICRTOF       	(0x00000001 << 24)
#define PRINTER_STATUS_MICRBOF     	   (0x00000001 << 25)

#define PRINTER_GS_PAPERROLL_NEAREND  1
#define PRINTER_GS_PAPERROLL_PAPER    2 
#define PRINTER_GS_PAPERROLL_TOF      3
#define PRINTER_GS_PAPERROLL_BOF      4
#define PRINTER_GS_DRAWER_OPEN        5
#define PRINTER_GS_SLIPPAPER_PRESENT  6

#define PRINTER_ABS_WAITINGONINSERT   1
#define PRINTER_ABS_SLIPINTSERTED	  2
#define PRINTER_ABS_WAITINGONEJECT	  4
#define PRINTER_ABS_SLIPEJECTED  	  8

#define PRINTER_BUFFER_STATUS_INPUT	  1
#define PRINTER_BUFFER_STATUS_OUTPUT  2

/** 
  \brief
 */ 
typedef struct tagPrintArea{
	int unit;
	int sx, sy, dx, dy;
}PrintArea;

/** 
  \brief
 */
typedef struct tagHeaderPrint{
	int  transID;
	int  UserID;
	int  StationID;
	char datetime[18];
	char msg[126];
}HeaderPrint;

/** 
  \brief
 */
typedef struct tagTransSummary{
   int   NumItems;
   float Subtotal;
   float Discount;
   float Tax;
   float Total;
}TransSummary;

/** 
  \brief
 */
typedef struct tagChargeSummary{
   float old_balance;
   float new_balance;
   float limit;
}ChargeSummary;


/** 
  \brief
 */
enum BhChequeLayoutConversion{
   CHK_LAYOUT_PAGE,
   CHK_LAYOUT_MM,
   CHK_LAYOUT_INCH
};

/** 
  \brief
 */
typedef struct tagcoord{
	int x;
	int y;
}coord;

/** 
  \brief
 */
typedef struct tagbcoord{
	unsigned char x[2];
	unsigned char y[2];
}bcoord;

/** 
  \brief
 */
typedef struct tagrcoord{
	float x;
	float y;
}rcoord;

/** 
  \brief
 */
typedef struct tagChequeLayout{
	int    x_dpi;
	int    y_dpi;
	rcoord size;
	rcoord date;
	rcoord year;
	rcoord number_amount;
	rcoord number_word1;
	rcoord number_word1_end;
	rcoord number_word2;
	rcoord payto;
}ChequeLayout;

/** 
  \brief
 */
typedef struct tagChequeUnitLayout{
	coord size;
	coord date;
	coord year;
	coord number_amount;
	coord number_word1;
	coord number_word1_end;
	coord number_word2;
	coord payto;
}ChequeUnitLayout;

/** 
  \brief
 */
typedef struct tagChequeByteUnitLayout{
	bcoord size;
	bcoord date;
	bcoord year;
	bcoord number_amount;
	bcoord number_word1;
	int	 num_word1_chars;
	bcoord number_word2;
	bcoord payto;
}ChequeByteUnitLayout;

/** 
  \brief
 */
typedef struct tagChequePrint{
	bool	 preLoaded;
   char   date[32];
	char   year[8];
	char   payto[64];
	char	 MajAmount[32];
	char	 MinAmount[32];
	double amount;
	char   endAccNum[32];
}ChequePrint;

/** 
  \brief
 */
#ifndef _DEFAULT_CHEQUE
#define _DEFAULT_CHEQUE
//static ChequeLayout StandardChequeUSPersonal={0,0,{152.4,69.83},{81,22},{109,22},{112,41},{23,31},{114,31},{7,39},{12,58}};
//static ChequeLayout StandardChequeUKPersonal={0,0,{152,71},{100,19},{145,19},{112,41},{10,42},{100,42},{10,51},{17,33}};

static ChequeLayout StandardChequeUSPersonal={0,0,{152,70},{72,51},{42,51},{30,40},{145,39},{38,39},{145,30},{137,58}};
static ChequeLayout StandardChequeUKPersonal={0,0,{152,71},{55,57},{11,57},{50,35},{10,42},{100,42},{10,51},{16,33}};
#define DEFAULT_NUM_CHK 2

/** 
  \brief
 */
static ChequeLayout BhStdCheque[DEFAULT_NUM_CHK]={
   StandardChequeUSPersonal,
   StandardChequeUKPersonal
};
#endif

/** 
  \brief
 */
typedef struct tagCreditPrint{
	bool	 bSigline;
	char   header[32];
   char   datetime[32];
	char   trans[16];
	char   station[16];
	char   user[16];
	char	 auth[32];
	char	 cardtype[16];
	char	 cardnum[32];
	double amount;
}CreditPrint;

/** 
  \brief
 */
enum BhPrinterFont{
   PF_PARAM_NUMBER,
   PF_PARAM_UNIT,
   PF_PARAM_XSIZE,
   PF_PARAM_YSIZE,
   PF_PARAM_CHARLINE
};

/** 
  \brief
 */
class CPrinterFont
{
private:
   int unit;
   int xsize, ysize;
   int charline;
   
public:
   CPrinterFont();
   CPrinterFont(CPrinterFont const&);
   CPrinterFont(int u, int x, int y, int cl);
   ~CPrinterFont();
   void operator=(CPrinterFont const&);
   bool operator==(CPrinterFont const&);
   bool operator<(CPrinterFont const&);
   
   void SetParameter(int, void *);
   void GetParameter(int, void *);
};

/** 
  \brief
 */
enum BhPrinterMechType{
   PM_TYPE_UNKNOWN=0,
   PM_TYPE_IMPACT,
   PM_TYPE_THERMAL,
   PM_TYPE_INKJET,
   PM_TYPE_LASER,
   PM_TYPE_WAXSUB
};

/** 
  \brief
 */
enum BhPrinterMechPaperType{
   PM_PAPERTYPE_UNKNOWN=0,
   PM_PAPERTYPE_RECIEPT,
   PM_PAPERTYPE_SLIP,
   PM_PAPERTYPE_JOURNAL,
   PM_PAPERTYPE_ENDORSE,
   PM_PAPERTYPE_LABEL,
   PM_PAPERTYPE_OTHER
};

/** 
  \brief
 */
enum BhPrinterMechCutterType{
   PM_CUTTERTYPE_NONE     = 1,
   PM_CUTTERTYPE_FULL     = 2,
   PM_CUTTERTYPE_PARTIAL  = 4,
   PM_CUTTERTYPE_AUTO     = 8
};

/** 
  \brief
 */
enum BhPrinterMechParam{
   PM_PARAM_TYPE,
   PM_PARAM_PAPERTYPE,
   PM_PARAM_XMOTIONUNIT,
   PM_PARAM_YMOTIONUNIT,
   PM_PARAM_XPAPERSIZE,
   PM_PARAM_YPAPERMINSIZE,
   PM_PARAM_YPAPERMAXSIZE,
   PM_PARAM_HASPAGEMODE,
   PM_PARAM_XDPI,
   PM_PARAM_YDPI,
   PM_PARAM_NUMFONTS,
   PM_PARAM_BIDIR,
   PM_PARAM_FONT,
   PM_PARAM_FONTSIZE,
   PM_PARAM_XCHARFONT,
   PM_PARAM_NUMCOLS,
   PM_PARAM_COLOUR,
   PM_PARAM_CUTTER,
   PM_PARAM_DEFCHARLINE
};

/** 
  \brief
 */
class CPrintMechanism{
private:
   BhPrinterMechType type; 
   BhPrinterMechPaperType papertype;
   int xpapersize;
   int ypaperminsize, ypapermaxsize;
   int xmotionunit, ymotionunit; 
   int xdpi, ydpi;
   
   bool bHasPageMode;
   bool bBidrectional;
   BhPrinterMechCutterType  cuttertype;
   
   std::list<CPrinterFont>   fonts;
   
   int curr_font;
   int numcolours;

protected:

public:
   CPrintMechanism();
   CPrintMechanism(CPrintMechanism const&);
   ~CPrintMechanism();
   void operator=(CPrintMechanism const&);
   bool operator==(CPrintMechanism const&);
   bool operator<(CPrintMechanism const&);
   
   void SetParameter(int, void *);
   void GetParameter(int, void *);
   int AddFont(CPrinterFont&);
};

/** 
  \brief
 */
typedef struct tagPrintCollect{
	unsigned char num;
	unsigned int  mask;
	unsigned int  data;
}PrintCollect;

/** 
  \brief
 */
enum BhPrinterProtocol{
   ESCPOS,
   DP8340,
   STAR,
   CITIZEN,
   PCOS,
   AXIHOM,
   PSTEMPLATE
};

/** 
  \brief
 */
enum MICRFontType{
   E13B=0,
   CMC7
};

/** 
  \brief
 */
enum BhPrintTimerType{
   PRNTIMER_SLIPINSERT,
   PRNTIMER_SLIPEJECT,
	PRNTIMER_CHQINSERT,
	PRNTIMER_CHQEJECT,
	PRNTIMER_MICRINSERT,
	PRNTIMER_MICREJECT
};

/** 
  \brief
 */
enum BhPrintPaperType{
   PRN_PAPERTYPE_UNKNOWN=0,
   PRN_PAPERTYPE_RECIEPT,
   PRN_PAPERTYPE_SLIP,
   PRN_PAPERTYPE_JOURNAL,
   PRN_PAPERTYPE_ENDORSE,
   PRN_PAPERTYPE_LABEL,
   PRN_PAPERTYPE_MICR,
	PRN_PAPERTYPE_CHEQUE
};
/*
enum BhPrintPaperStatusType{
   PRN_PAPERTYPE_UNKNOWN=0,
   PRN_PAPERTYPE_RECIEPT,
   PRN_PAPERTYPE_SLIPSELECT,
   PRN_PAPERTYPE_SLIPINSERT,
   PRN_PAPERTYPE_SLIPEJECT,
   PRN_PAPERTYPE_JOURNAL,
   PRN_PAPERTYPE_ENDORSE,
   PRN_PAPERTYPE_LABEL,
   PRN_PAPERTYPE_MICR,
	PRN_PAPERTYPE_CHEQUE
};
*/

/** 
  \brief
 */
enum BhPrintType{
   PRNCMD_SETUP,
   PRNCMD_INTIALIZE,
   PRNCMD_NVBITMAP,
   PRNCMD_PREPRINT,
   PRNCMD_HEADER,
   PRNCMD_ITEM,
   PRNCMD_TRANSSUMMARY,
	PRNCMD_PAYITEM,
   PRNCMD_PAYSUMMARY,
	PRNCMD_CHANGE,
   PRNCMD_CHARGE,
   PRNCMD_FOOTER,
   PRNCMD_POSTPRINT,
   PRNCMD_SLIP_PREPRINT,
   PRNCMD_SLIP_HEADER,
   PRNCMD_SLIP_ITEM,
   PRNCMD_SLIP_FOOTER,
   PRNCMD_SLIP_POSTPRINT,
	PRNCMD_SLIP_CANCEL,
   PRNCMD_LABEL_PREPRINT,
   PRNCMD_LABEL_ITEM,
   PRNCMD_LABEL_POSTPRINT,
   PRNCMD_JRN_PREPRINT,
   PRNCMD_JRN_ITEM,
   PRNCMD_JRN_POSTPRINT,
   PRNCMD_END_PREPRINT,
   PRNCMD_END_ITEM,
   PRNCMD_END_POSTPRINT,
   PRNCMD_CHQ_PREPRINT,
   PRNCMD_CHQ_ITEM,
   PRNCMD_CHQ_POSTPRINT,
   PRNCMD_CHQ_LOADTIMER,
   PRNCMD_CHQ_PRINT,
   PRNCMD_CHQ_EJECTTIMER,
   PRNCMD_BARCODE,
   PRNCMD_CDISPCLEAR,
   PRNCMD_CDISPCMD,
   PRNCMD_CDISPPRESALE,
   PRNCMD_CDISPPOSTSALE,
	PRNCMD_CDISPIDLE,
   PRNCMD_CDISPLINE1,
   PRNCMD_CDISPLINE2,
   PRNCMD_CDISP,
   PRNCMD_CASHD,
   PRNCMD_CASHDTIMER,
   PRNCMD_MICR_PREREAD,
   PRNCMD_MICR_READ,
   PRNCMD_MICR_STATUS,
   PRNCMD_MICR_DATA,
   PRNCMD_MICR_POSTREAD,
	PRNCMD_MICR_CANCEL,
	PRNCMD_CREDIT,
	PRNCMD_INVOICE,
   PRNCMD_TEMPLATE,
   PRNCMD_RAW
};

/** 
  \brief
 */
enum BhPrintParam{
   PRN_PARAM_PAGEMODE,
   PRN_PARAM_DEFXRES,
   PRN_PARAM_DEFYRES,
   PRN_PARAM_CASHDRAWER,
   PRN_PARAM_CASHDRAWERSB,
   PRN_PARAM_CASHDRAWERSBPERIOD,
   PRN_PARAM_CUSTDISP,
   PRN_PARAM_MICR,
   PRN_PARAM_MICRFONTYPE,
   PRN_PARAM_CHARSET,
   PRN_PARAM_RECCHARLINE,
   PRN_PARAM_RECFONT,
   PRN_PARAM_JRNCHARLINE,
   PRN_PARAM_JRNFONT,
   PRN_PARAM_SLIPCHARLINE,
   PRN_PARAM_SLIPFONT,
   PRN_PARAM_CHEQUE,
   PRN_PARAM_CHEQUEROTATE,
   PRN_PARAM_CHEQUEENDORSE,
   PRN_PARAM_CUSTDISPTYPE,
   PRN_PARAM_CUSTDISPX,
   PRN_PARAM_CUSTDISPY,
	PRN_PARAM_NVRAM,
	PRN_PARAM_NVRAMSIZE,
	PRN_PARAM_BARCODE,
	PRN_PARAM_AUTOCUTTER,
   PRN_PARAM_CREDITSLIP,
	PRN_PARAM_AUTOPOLE,
	PRN_PARAM_NVRAM_HDRLOGO,
   PRN_PARAM_NVRAM_FTRLOGO,
   PRN_PARAM_NVRAM_SLIPLOGO,
   PRN_PARAM_NVRAM_HDRMSG,
   PRN_PARAM_NVRAM_FTRMSG,
   PRN_PARAM_NVRAM_RECHDRLAYOUT,
   PRN_PARAM_NVRAM_ITEMLAYOUT,
   PRN_PARAM_NVRAM_PAYSUMLAYOUT,
   PRN_PARAM_NVRAM_TRANSSUMLAYOUT,
   PRN_PARAM_NVRAM_SLIPHDRLAYOUT,
   PRN_PARAM_NVRAM_SLIPITEMLAYOUT,
   PRN_PARAM_NVRAM_SLIPPAYSUMLAYOUT,
   PRN_PARAM_NVRAM_SLIPTRANSSUMLAYOUT,
   PRN_PARAM_NVRAM_JRNHDR,
   PRN_PARAM_NVRAM_JRNITEM,
   PRN_PARAM_NVRAM_JRNFTR,
   PRN_PARAM_NVRAM_CHPAYTO,
   PRN_PARAM_NVRAM_MAJAMOUNT,
   PRN_PARAM_NVRAM_MINAMOUNT,
   PRN_PARAM_NVRAM_CDIDLEMSG,
   PRN_PARAM_NVRAM_CDPRESALE,
   PRN_PARAM_NVRAM_CDPOSTSALE,
   PRN_PARAM_NVRAM_ROLLCNT,
   PRN_PARAM_NVRAM_MICRCNT
};

/** 
  \brief
 */
enum BhPrinterNvRam{
   PRN_NVRAM_HDRLOGO=1,
   PRN_NVRAM_FTRLOGO,
   PRN_NVRAM_SLIPLOGO,
   PRN_NVRAM_HDRMSG=40,
   PRN_NVRAM_FTRMSG,
   PRN_NVRAM_RECHDRLAYOUT,
   PRN_NVRAM_ITEMLAYOUT,
   PRN_NVRAM_PAYSUMLAYOUT,
   PRN_NVRAM_TRANSSUMLAYOUT,
   PRN_NVRAM_SLIPHDRLAYOUT,
   PRN_NVRAM_SLIPITEMLAYOUT,
   PRN_NVRAM_SLIPPAYSUMLAYOUT,
   PRN_NVRAM_SLIPTRANSSUMLAYOUT,
   PRN_NVRAM_JRNHDR,
   PRN_NVRAM_JRNITEM,
   PRN_NVRAM_JRNFTR,
   PRN_NVRAM_CHPAYTO,
   PRN_NVRAM_MAJAMOUNT,
   PRN_NVRAM_MINAMOUNT,
   PRN_NVRAM_CDIDLEMSG,
   PRN_NVRAM_CDPRESALE,
   PRN_NVRAM_CDPOSTSALE,
   PRN_NVRAM_ROLLCNT,
   PRN_NVRAM_MICRCNT
};


#define PRN_INSTREAM_POLLMICR   1
#define PRN_INSTREAM_POLLCHEQUE 2

#define PRN_NOMOREDATA	40
#define PRN_MOREDATA	   41

/** 
  \brief
 */
typedef struct tagPrnNVMsg{
	unsigned char header;
	unsigned char ident;
	unsigned char status;
	char			  data[81];
}PrnNVMsg;

/** 
  \brief
 */
typedef struct tagPrnNVRam{
	unsigned char key1;
	unsigned char key2;
	unsigned char size;
	unsigned char status;
	char			  data[1];
}PrnNVRam;

/** 
  \brief
 */
class CPrinter : public CBhDevice
{
private:
	int               printer_status;
	BhPrinterProtocol protocol;
	bool				   bRealTimeStatusIssued;
	bool				   bWaitingPaperInsert;
   bool              bCreditCut;
   bool              bCreditSlip;
   bool              bAutoThread;
	
	unsigned int reccharline;
   unsigned int jrncharline;
   unsigned int slipcharline;
   unsigned int recfont;
   unsigned int jrnfont;
   unsigned int slipfont;
	unsigned int charsize;
   unsigned int max_line;

	char *linebuffer;
   
   // features for real time use
   bool bCustomerDisplay;
	bool bAutoCustomerDisplay;
   int  custdisp_type;
   bool bCashDrawer;
   bool bPageMode;
	bool bNVRAM;
	bool bMICR;
   bool bAutoCutter;
	bool bBarcode;
   bool bChequePrint;
   bool bChequeRotate;
   bool bEndorsement;
   
   bool bCashDrawerStatusBack;
   int  cashd_statusperiod;
   
   unsigned int  nvramsize;
   unsigned int  x_res;
   unsigned int  y_res;
   unsigned int  custdisp_x;
   unsigned int  custdisp_y;
   MICRFontType  mircft;
	ChequeByteUnitLayout chklayout;
   
   int          tbuffsize;
   char         tbuff[128];
   
   unsigned char  tprinttype, num_copies;
   unsigned char  num_top, num_end, num_cont, num_sheets;
   unsigned char  top_yinc, end_yinc, cont_yinc;
   char          *topbuff, *contbuff, *endbuff;
   char          *custdisp_line1, *custdisp_line2;
   int            top_len, end_len, cont_len;
   FILE          *tp, *cp, *ep;
	
	bool                    bAbsWatcher;
   BhPrinterStatusMethod   statusmethod;
   BhPrinterStatusType     statustype;

protected:
   int Setup(char *d, int len);
	int Initialize(char *d, int len);
   int SetCharSet(int);
	int PrePrint(char *d, int len);
	int PrintHeader(HeaderPrint *);
	int PrintItem(CSaleItem *);
	int PrintTransSummary(TransSummary *);
	int PrintPayItem(int, CPayItem *);
	int PrintChange(double *);
   int PrintCharge(ChargeSummary *);
	int PrintPaySummary(CTransItems *);
	int PrintFooter(char *d, int len);
	int PostPrint(char *d, int len);
   int SlipPrePrint(char *d, int len);
   int SlipHeader(char *, int);
	int SlipItem(CSaleItem *);
	int SlipFooter(char *, int);
	int SlipPostPrint(char *d, int len);
	int SlipCancel();
   int LabelPrePrint(char *d, int len);
	int LabelItem(CSaleItem *);
	int LabelPostPrint(char *d, int len);
   int JournalPrePrint(char *d, int len);
	int JournalItem(CSaleItem *);
	int JournalPostPrint(char *d, int len);
   int EndorsementPrePrint(char *d, int len);
	int EndorsementItem(CTransItems *);
	int EndorsementPostPrint(char *d, int len);
	int ChequePrePrint(char *d, int len);
	int ChequeItem(ChequePrint *);
	int ChequePostPrint(char *d, int len);
   int ChequePrintItem(ChequePrint *);
	int PrintCredit(CreditPrint *);
	int MicrPreread(char *d, int len);
	int MicrRead();
	unsigned char MicrStatus();
	int MicrCancel();
   int MicrData(BhChequeFormattedData *);
	int MicrPostread(char *d, int len);
	int PrintBarcode();
   int PrintCustDispCmd(int);
	int PrintCustDisp(unsigned char disp, int x, int y, CSaleItem *msg);
	int PrintCustDisp(bool, char *);
   int PrintCustDisp(int, int);
   int PrintCustDisp(char *, char *);
	int FireCashDrawer(int drawer);
	int PaperCut(unsigned char type, unsigned char feed);
   
   int  LoadSlip();
   bool CheckSlip();
   void CancelSlip();
   void EjectSlip();
   
   void StatusThread(BhPrinterStatusType type);
	
	int  ChequeUnitToByte(ChequeUnitLayout *, ChequeByteUnitLayout *);
   void SpellMoney(double, char *, char *, char *);
	void JustifyLine(int, char *, char *);
   
   int GetPageVariables(int);
   int ReplaceVariableInstances(int f, int numc, CSaleItem &si);
   int ReplaceFixedInstances(int f, int numc, int sheetnum, int sheetof, CTransItems &trans, BhCustomerItem *cust);
   int ReplaceTag(int f, char *tag, char *rep);
   
   sigc::signal<void, BhPrinterStatusEvent, void *> status_signal;
   sigc::signal<void, BhPrinterErrorEvent, void *> error_signal;
   
public:
   CPrinter(BhDevType);
	~CPrinter();
	
	bool PrinterExists();
	
	int  SetParam(BhPrintParam, void *);
	int  GetParam(BhPrintParam, void *);
	int  QuickParse(char *s);
	
   int Output(int, void *);
	int Input(int, void *, bool);

	int ReadNVRAM(char, char, char *);
   int WriteNVRam(int, char, char, unsigned char *);
	int NVRam(bool bUsed);
   
   int OpenTemplate(char *toppage, char *contpage, char *endpage);

	void TabSet(int flags);
	int  ChequeLayoutToUnit(ChequeLayout *, ChequeUnitLayout *, BhChequeLayoutConversion);
	int  ChequeLayoutToUnit(ChequeLayout *, BhChequeLayoutConversion);
	int  PrintZTape(HeaderPrint *, BhServiceTransSum *);

	int Print(BhPrintType, int, void *);
   int PrintTemplate(CTransItems &trans, BhCustomerItem *cust, int numcopies);
	unsigned int GetPrinterStatus(unsigned int type, BhPrinterStatusMethod method);
	int GetPaperStatus(BhPrintPaperType type, bool bPresent);
   BhPrinterStatusType GetStatusType() { return statustype; }
   BhPrinterStatusMethod GetStatusMethod() { return statusmethod; }
	void Lock();
	void Release();
   
   void Signal(BhPrinterStatusEvent, void *);
   void Error(BhPrinterErrorEvent, void *);
   sigc::signal<void, BhPrinterStatusEvent, void *> signal_status() { return status_signal; }
   sigc::signal<void, BhPrinterErrorEvent, void *> signal_error() { return error_signal; }
};

namespace parser{

#define PRNLO_HAVEPAGEMODE 1
#define PRNLO_HEADER	 	   2
#define PRNLO_ITEM	 	   3
#define PRNLO_FOOTER	 	   4

/** 
  \brief
 */
enum BhPageAttribute{
   PRN_PAGE_TOPMARGIN,
   PRN_PAGE_BOTTOMMARGIN,
   PRN_PAGE_LEFTMARGIN,
   PRN_PAGE_RIGHTMARGIN,
   PRN_PAGE_LENGTHLINE,
   PRN_PAGE_LENGTHUNIT,      //! page length in millimetres
   PRN_PAGE_ORIGINX,         //! set origin of x in points
   PRN_PAGE_ORIGINY,         //! set origin of y in points
   PRN_PAGE_SIZEX,           //! set size of x in points
   PRN_PAGE_SIZEY,           //! set size of y in points
   PRN_PAGE_ENDX,            //! set bottom right coordinate of x in points
   PRN_PAGE_ENDY             //! set bottom right coordinate of y in points
};

/** 
  \brief
 */
enum BhCharAttribute{
   PRN_CHAR_SET        = 1,      // character set
   PRN_CHAR_DOTPITCH   = 2,
   PRN_CHAR_SPACING    = 4,
   PRN_CHAR_MAGNIFIY   = 8
};

/** 
  \brief
 */
enum BhPrintAttribute{
   PRN_ATTRIB_EMPHASIS   = 1,
   PRN_ATTRIB_UNDERLINE  = 2,
   PRN_ATTRIB_REVERSE    = 4,
   PRN_ATTRIB_DBLSTRIKE  = 8,
   PRN_ATTRIB_UPSIDEDOWN = 16,
   PRN_ATTRIB_ROTATE     = 32
};

/** 
  \brief
 */
class CPrinterDefinitionParser
{
private:
   Glib::ustring   filename;
   xmlpp::Document document;
   xmlpp::Element* nodeRoot;
   
protected:
   Glib::ustring prnname;
   Glib::ustring manufacturer;
   Glib::ustring model;
   BhPrinterProtocol protocol;
   int   nvramsize;
   int   buffersize;
   bool  bCustomerDisplay;
   //int   custdisp_type;
   //int   custdisp_x;
   //int   custdisp_y;
   //bool  bCashDrawer;
   int   numcashdrawer;
   bool  bMICR;
   bool  bCutter;
   
public:
   CPrinterDefinitionParser();
   ~CPrinterDefinitionParser();
   int Open(Glib::ustring &fname, CPrinter&);
   int Save(Glib::ustring &fname, CPrinter&);
   int ParsePrinter(xmlpp::Node *node, CPrinter&);
   int ParseMechanism(xmlpp::Node *node, CPrinter&);
   int ParseCutter(xmlpp::Node *node, CPrintMechanism&);
   int ParseColour(xmlpp::Node *node, CPrintMechanism&);
   int ParseFont(xmlpp::Node *node, CPrintMechanism&);
   xmlpp::Node GetCommandNode();
};

/** 
  \brief
 */
class CPrintLayoutParser
{
private:
   Glib::ustring filename;

protected:
   void ParsePrinter(xmlpp::Node node, CPrinter& prn);
   void ParseSetup(xmlpp::Node node, CPrinter& prn);
   void ParsePrePrint(xmlpp::Node node, CPrinter& prn);
   void ParseHeader(xmlpp::Node node, CPrinter& prn);
   void ParseItem(xmlpp::Node node, CPrinter& prn);
   void ParseTransSum(xmlpp::Node node, CPrinter& prn);
   void ParseFooter(xmlpp::Node node, CPrinter& prn);
   void ParsePostPrint(xmlpp::Node node, CPrinter& prn);
   void ParseEndorsement(xmlpp::Node node, CPrinter& prn);
   void ParseCheque(xmlpp::Node node, CPrinter& prn);
   void ParseSlip(xmlpp::Node node, CPrinter& prn);
   void ParseLabel(xmlpp::Node node, CPrinter& prn);
   void ParseBarcode(xmlpp::Node node, CPrinter& prn);
   void ParseTextAttribute(xmlpp::Node node, unsigned char *buff);
   void ParseInJustify(xmlpp::Node node, CPrinter& prn, unsigned int flags);
   void ParseTextAttribute(xmlpp::Node node);
   void ParseBitmap(xmlpp::Node node);
   void ParseJustify(xmlpp::Node node);
   void ParseFont(xmlpp::Node node);
   unsigned char LowByteValue(int i);
   unsigned char HiByteValue(int i);
   void ParseColour(xmlpp::Node node);
   int buffcat(unsigned char *, unsigned char);

public:   
   CPrintLayoutParser();
   ~CPrintLayoutParser();
   int Open(Glib::ustring &fname, CPrinter& prn);
   bool GetOption(unsigned int param, void *data);
   bool SetOption(unsigned int param, void *data);
   int GetError();
   char *GetErrorString(int err);
};

}

/*	}@ 
	}@ */
}

/* }@ */

#endif	// _CPRINTER_H_
